/*
 *  TOPPERS/JSP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Just Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 * 
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation 
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 * 
 *  @(#) $Id: component.h,v 1.4 2003/11/10 07:51:14 monaka Exp $
 */


// $Header: /cvsroot/toppers-utils/if40/cfg/base/component.h,v 1.4 2003/11/10 07:51:14 monaka Exp $

#ifndef __COMPONENT
#define __COMPONENT

#ifdef _MSC_VER
	#pragma warning(disable:4786)
#endif

#include <list>
#include <map>
#include <string>
#include <typeinfo>

#include "defs.h"
#include "except.h"
#include "directorymap.h"

#define DEFAULT_PARAMETER " "
#define FILENAME_PERSISTENTCONTAINER "settings.dat"

//------

class Component
{
public:
	enum tagBarrier   { EXTENSION, IGNORE_UNKNOWN_API, NO_SUPPORT };

		//μ¹Խ
	enum tagProcessStage
	{
		INITIALIZE,					//Ȥˤư

		BEFORE_PARSE,				//ѡ
		PARSER,						//ѡ
		AFTER_PARSE,				//ѡ

		PREDECESSOR,				//Ѥ
		GENERAL_PURPOSE,			//
		SUCCESSOR,					//Ѥθ

		BEFORE_CODEGENERATION,		//
		CODEGENERATOR,				//
		AFTER_CODEGENERATION,		//

		FINALIZE
	};

private:
	static std::multimap<int,Component *> & componentList(void);
	static std::list<Component *> &         activatedComponentList(void);
	static Directory                        persistentContainer;

	static void  loadPersistentContainer(Directory &);
	static void  storePersistentContainer(Directory &);

protected:
	static int barrierFlag;

	static std::string & bootBanner(void);

    static Component * findComponent(const char *);

    static bool        findOption (Directory &, const char *, const char * = 0, std::string * = 0);
	static bool        checkOption(Directory &, const char *, const char * = 0);
	static Directory * getOption  (Directory &, const char *, const char * = 0, bool = true);
	static Directory * mergeOption(Directory &, const char *, const char *);

	static void enable(enum tagBarrier);
	static void disable(enum tagBarrier);

	static Directory &  getPersisitentContainer(void);
	static std::string  getPersistentString   (const std::string & key, std::string _default = "");
	static int			getPersistentValue    (const std::string & key, int _default = 0);
	static std::string  setPersistentString   (const std::string & key, std::string _default = "");
	static int          setPersistentValue    (const std::string & key, int _default = 0);
	static std::string  updatePersistentString(const std::string & key, std::string _default = "");
	static int          updatePersistentValue (const std::string & key, int _default = 0);

	static void dumpContainerSnapshot(Directory &, bool = false);
    
    virtual void parseOption(Directory &) {};    
    virtual void body(Directory &)        {};
	virtual void onFatalExit(Directory &) {};

public:
	Component(int order = GENERAL_PURPOSE);
    virtual ~Component(void);

    void activateComponent(bool = true);

	static bool isEnabled(enum tagBarrier);
	static void barrier(enum tagBarrier, const Message & reason);

    static void parseOption(int, char **, Directory &, const char * = NULL);
    static void executeComponents(Directory &);
	static void fire_onFatalExit(Directory &);
};

#include <iostream>

inline void Component::enable(enum tagBarrier bf)
{	barrierFlag |= 1 << (int)bf;	}

inline void Component::disable(enum tagBarrier bf)
{	barrierFlag &= ~(1 << (int)bf);	}

inline bool Component::isEnabled(enum tagBarrier bf)
{	return (barrierFlag & (1 << (int)bf)) != 0;	}

inline void Component::barrier(enum tagBarrier bf, const Message & reason)
{
	if((barrierFlag & (1 << (int)bf)) == 0)
		throw Exception("The operation detailed below is not permitted.\n %s\n","ϵĤƤޤ\n %s\n").format((const char *)reason);

	if(!((std::string &)reason).empty())
		VerboseMessage("Configurator allows the operation below\n %s\n","Ĥޤ\n %s\n").format((const char *)reason);
}

inline Directory & Component::getPersisitentContainer(void)
{	return persistentContainer;   }

inline std::string Component::getPersistentString(const std::string & key, std::string _default)
{	return persistentContainer.findChild(key)->toString(_default);   }

inline std::string Component::setPersistentString(const std::string & key, std::string value)
{
	(*persistentContainer.openChild(key)) = value;   
	return value;
}

inline std::string Component::updatePersistentString(const std::string & key, std::string value)
{
	Directory * node;
	node = persistentContainer.openChild(key);
	if(node->toString().compare(value) != 0)
		*node = value;
	return value;
}

inline int Component::getPersistentValue(const std::string & key, int _default)
{	return persistentContainer.findChild(key)->toInteger(_default);   }

inline int Component::setPersistentValue(const std::string & key, int value)
{	return *persistentContainer.openChild(key) = value;   }

inline int Component::updatePersistentValue(const std::string & key, int value)
{
	Directory * node;
	node = persistentContainer.openChild(key);
	if(node->toInteger() != value)
		*node = value;
	return value;
}

#endif
