/*
 *  TOPPERS/JSP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Just Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 * 
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation 
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 * 
 *  @(#) $Id: mpstrstream.h,v 1.4 2003/11/10 07:51:14 monaka Exp $
 */

// $Header: /cvsroot/toppers-utils/if40/cfg/base/mpstrstream.h,v 1.4 2003/11/10 07:51:14 monaka Exp $

#ifndef __MPFSTREAM
#define __MPFSTREAM

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable:4786)
	#pragma warning(disable:4290)
#endif

#include "except.h"
#include "garbage.h"

#include <fstream>
#include <sstream>
#include <list>
#include <string>

class mpstrstream : public Garbage
{
public:
	enum { DEFAULT = 0, NO_SERIALIZE = 1 };
	enum tagPosition { BEGIN, END };
	struct tagStreamPartInfo
	{
		std::string	       name;
		std::stringstream  stream;
		int                flags;
	};

protected:
	std::string										filename;
	std::list<struct tagStreamPartInfo *>           container;
	std::list<struct tagStreamPartInfo *>::iterator current;
	bool											fileoutput;

	std::list<struct tagStreamPartInfo *>::iterator findpart(const char * partname) const;

public:
	mpstrstream(void);
	mpstrstream(const std::string &);
	mpstrstream(const char * filename);
	
	virtual ~mpstrstream(void);

	bool                operator !  (void) const;
	std::stringstream & operator *  (void) const;
	std::stringstream & operator [] (const char *);
	std::stringstream & operator [] (unsigned int);
	std::stringstream & operator [] (enum tagPosition);
	std::stringstream & operator ++ (void);
	std::stringstream & operator -- (void);

	bool                is_valid(void)      const;
    const std::string & getStreamName(void) const;
	const std::string &	getPartName(void)   const;
	std::stringstream & getStream(void)     const;

	std::stringstream & moveto(const std::string & partname) throw(Exception);
	std::stringstream & moveto(const char * partname)        throw(Exception);
	std::stringstream & moveto(unsigned int offset);
	std::stringstream & moveto(enum tagPosition pos);
	std::stringstream & movenext(void);
	std::stringstream & moveprev(void);

	std::stringstream & createpart(const char * partname, const char * second = 0, ... );
	std::stringstream & insertpart(const char * partname, const char * successor = 0)    throw(Exception);
	void                erasepart(const char * partname);
	void                movepart(const char * partname, const char * successor = 0)      throw(Exception);
	bool                is_extantpart(const char * partname) const;

	void                assignFilename(const std::string &);
	void                assignFilename(const char *);

	void                serialize(void)           throw(Exception);
	std::ostream &      serialize(std::ostream &);

	void                enableFileoutput(void);
	void                disableFileoutput(void);

	int                 setf(const char * partname, int);
	int                 setf(int);

	template<class T>
	std::stringstream & operator << (const T & src) { (*current)->stream << src; return (*current)->stream; };

};

inline mpstrstream::mpstrstream(void) : fileoutput(false)
{	current = container.end();	}

inline mpstrstream::mpstrstream(const std::string & src) : filename(src), fileoutput(true)
{	current = container.end();	}

inline mpstrstream::mpstrstream(const char * file) : filename(file), fileoutput(true)
{	current = container.end();	}

inline const std::string & mpstrstream::getStreamName(void) const
{	return filename;	}

inline const std::string & mpstrstream::getPartName(void) const
{	return (*current)->name;	}

inline std::stringstream & mpstrstream::getStream (void) const
{	return (*current)->stream;	}

inline std::stringstream & mpstrstream::operator * (void) const
{	return getStream();	}

inline bool mpstrstream::is_valid(void) const
{	return !(container.end() == current);	}

inline bool mpstrstream::operator ! (void) const
{	return is_valid();	}

inline std::stringstream & mpstrstream::operator [](const char * partname)
{	return moveto(partname);	}

inline std::stringstream & mpstrstream::operator [](unsigned int offset)
{	return moveto(offset);	}

inline std::stringstream & mpstrstream::operator [](enum tagPosition pos)
{	return moveto(pos);	}

inline std::stringstream & mpstrstream::operator ++ (void)
{	return movenext();	}

inline std::stringstream & mpstrstream::operator -- (void)
{	return moveprev();	}

inline std::stringstream & mpstrstream::moveto(const std::string & partname) throw(Exception)
{	return moveto(partname.c_str());	}

inline std::stringstream & mpstrstream::movenext(void)
{	++ current; return (*current)->stream;	}

inline std::stringstream & mpstrstream::moveprev(void)
{	-- current; return (*current)->stream;	}

inline void mpstrstream::assignFilename(const std::string & src)
{	filename.assign(src);	}

inline void mpstrstream::assignFilename(const char * src)
{	filename.assign(src);	}

inline void mpstrstream::enableFileoutput(void)
{	fileoutput = true;	};

inline void mpstrstream::disableFileoutput(void)
{	fileoutput = false; };

inline int mpstrstream::setf(const char * partname, int flag)
{
	this->movepart(partname);
	return setf(flag);
}

inline bool mpstrstream::is_extantpart(const char * partname) const
{	return container.end() != findpart(partname);	}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#endif
