/*
 *  TOPPERS/JSP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Just Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2002 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 * 
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation 
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 * 
 *  @(#) $Id: jsp_check.cpp,v 1.3 2003/11/10 07:51:15 monaka Exp $
 */

// $Header: /cvsroot/toppers-utils/if40/cfg/jsp/jsp_check.cpp,v 1.3 2003/11/10 07:51:15 monaka Exp $

#include "base/defs.h"
#include "jsp/jsp_defs.h"
#include "base/component.h"
#include "base/filecontainer.h"
#include <fstream>
#include <iomanip>

class ConfigurationChecker : public Component
{
public:

	enum tagCheckLevel
	{
		UNKNOWN    = 0,
		LAZY       = 1,		/* ̿Ū (Ԥ᤮¤ɻ)  */
		STANDARD   = 2,		/* ITRONͤϰ (¤) */
		TOPPERS    = 4,		/* TOPPERS/JSPϰ          */
		RESTRICTED = 8,		/* Ҥüޤ           */

		NORMAL     = 8
	};

protected:
	enum tagCheckLevel current_level;

	unsigned int error_count;
	std::string banner;

	void set_banner(Directory &, Formatter , const char *, int);
	void notify(enum tagCheckLevel, Formatter , bool = true);

	bool check_taskblock(Directory &, filecontainer *);
	bool check_semaphoreblock(Directory &, filecontainer *);
	bool check_eventflagblock(Directory &, filecontainer *);
	bool check_dataqueueblock(Directory &, filecontainer *);
	bool check_mailboxblock(Directory &, filecontainer *);
	bool check_fixed_memorypoolblock(Directory &, filecontainer *);
	bool check_cyclic_handlerblock(Directory &, filecontainer *);
	bool check_interrupt_handlerblock(Directory &, filecontainer *);
	bool check_exception_handlerblock(Directory &, filecontainer *);

	virtual void parseOption(Directory &);
	virtual void body(Directory &);

public:
	ConfigurationChecker(void);
};

//------------------------------------------------------
using namespace std;

ConfigurationChecker  instance_of_ConfigurationChecker;

//------------------------------------------------------


#define LoadVal(x,y,z) x = 0; container->load_variable(&x, sizeof(x), y, z)
#define Load(x,y) container->load_variable(&(x), sizeof(x), y)

ConfigurationChecker::ConfigurationChecker(void)
{
	bootBanner() = "--- TOPPERS/JSP Configuration Checker (ver 2.3) ---";
}

void ConfigurationChecker::set_banner(Directory & container, Formatter object, const char * type, int id)
{
	string objectname;
	Directory * node;
	char buffer[32];


	banner = string("    ") + object.str() + " : ";

	objectname = filecontainer::get_instance()->get_objectname(id, type);
	sprintf(buffer, "id = %d", id);

	if(objectname.empty())
		banner += buffer;
	else
		banner += objectname + " (" + buffer + ") ";

	node = container.findChild(OBJECTTREE, type, objectname.c_str(), "position", NULL);
	if(node != 0)
		banner += string(" at ") + node->toString();

	if(VerboseMessage::getVerbose())
	{
		cout << banner << endl;
		banner.erase();
	}
}

void ConfigurationChecker::notify(enum tagCheckLevel level, Formatter msg, bool error)
{
	if((level & current_level) != 0)
	{

		if(!banner.empty())
		{
			cout << banner << endl;
			banner.erase();
		}

		cout << "      ";

		if(error)
		{
			cout << Message("[ Error ] ","[顼] ");
			error_count ++;
		}else
			cout << Message("[Warning] ","[ ٹ ] ");
		cout << msg << endl;
	}
}

	/*
	 * ֥Ȥ˴ؤ륨顼
	 */

bool ConfigurationChecker::check_taskblock(Directory & parameter, filecontainer * container)
{
	unsigned int id;
	unsigned int maxpri;
	unsigned int minpri;
	unsigned int work;
	unsigned int num;
	struct variable_information tinib_table;
	unsigned int old_error_count = error_count;

	Message object("Task","");

	if(!Load(num, "_kernel_tmax_tskid"))
		ExceptionMessage(
			"Internal error: Unknown symbol (Probably, Symbol table was stripped)",
			"顼: ʥܥ̾ (¹Էܥơ֥ޤǤʤǽޤ)").throwException();

	if(num < 1)
	{
		notify(RESTRICTED,
			Message("  [Task] : No tasks created\n","  [] : ֥Ȥޤ\n"));
		return true;
	}

	tinib_table = container->get_variable_information("_kernel_tinib_table");
	maxpri = container->get_variable_information("TMAX_TPRI").value;
	minpri = container->get_variable_information("TMIN_TPRI").value;
	VerboseMessage("% object : % items\n","%֥ : % \n") << object << num;
	for(id = 1; id <= num; id++)
	{
		set_banner(parameter, object, TASK, id);

			/*
			 *  °å
			 */
		LoadVal(work, tinib_table.address, "task_initialization_block::tskatr");

			// °ͤ TA_HLNG|TA_ASM|TA_ACT ʳͤȤäƤ
		if((work & ~0x3) != 0)
			notify( STANDARD,
				Message("Illegal task attribute (It should be ((TA_HLNG||TA_ASM)|TA_ACT))",
						"ʥ° ((TA_HLNG||TA_ASM)|TA_ACT)ʳ"));

			// °ͤ TA_ASM ޤޤƤ
		if((work & 0x1) != 0)
			notify( RESTRICTED,
				Message("TA_ASM specified as task attribute takes no effect.",
						"°TA_ASMꤵƤ"));

			/* ưϤ0 */
		LoadVal(work, tinib_table.address, "task_initialization_block::task");
		if(work == 0)
			notify(RESTRICTED,
				Message("The address of task routine is equal to zero.",
						"Ϥ0ꤵƤޤ"));

			/*
			 *  ͥ٥å
			 */

		LoadVal(work, tinib_table.address, "task_initialization_block::ipriority");

			// ͥ٤ȺǾͥ٤꤬
		if(maxpri < minpri)
			notify(LAZY,
				Message("Illegal Priority Settings found (TMAX_TPRI(%) < TMIN_TPRI).",
						"ͥ٤ͥ(%)ĶƤ") << minpri);

			// ͥ٤ϰϤ[Ǿͥ, ͥ]ϰϤĶƤ
		if((signed)work > (signed)(maxpri - minpri))
			notify(TOPPERS,
				Message("Initial priority is greater than maximum priority (%).",
						"ͥ٤ͥ(%)ĶƤ") << maxpri);
		if((signed)work < 0)
			notify(STANDARD,
				Message("Initial priority is less than the minimum priority (%).",
						"ͥ٤ͥ(%)򲼲") << minpri);

			/*
			 *   㳰°å
			 */

		LoadVal(work, tinib_table.address, "task_initialization_block::texatr");

			// °ͤTA_HLNG or TA_ASMǤʤ
		if((work & ~0x3) != 0)
			notify(STANDARD,
				Message("Task exception routine has an illegal attribute specifier.",
						"㳰̵°(TA_HLNG,TA_ASMʳ) ꤵƤޤ"));

			// °ͤ TA_ASM ޤޤƤ
		if((work & 0x1) != 0)
			notify( RESTRICTED,
				Message("TA_ASM, specified as texatr, does not always take effect.",
						"㳰TA_ASMꤵƤޤ"));

//		LoadVal(work, tinib_table.address, "task_initialization_block::texhdr");
//		if(work == 0)
//			notify(RESTRICTED,
//				Message("NULL pointer is specified as an texrtn address.",
//						"㳰ϥɥϤNULLǤ"));

			/*
			 *   åå
			 */

			// å0
		LoadVal(work, tinib_table.address, "task_initialization_block::stksz");
		if(work == 0)
			notify(RESTRICTED,
				Message("Stack size is equal to zero.",
						"å0ꤵƤޤ"));

			// åϤ0
		LoadVal(work, tinib_table.address, "task_initialization_block::stk");
		if(work == 0)
			notify(RESTRICTED,
				Message("The address of task stack is equal to zero.",
						"åϤ0ꤵƤޤ"));

		tinib_table.address += tinib_table.size;
	}

	return old_error_count == error_count;
}

bool ConfigurationChecker::check_semaphoreblock(Directory & parameter, filecontainer * container)
{
	unsigned int id;
	unsigned int num;
	unsigned int work;
	unsigned int work2;
	unsigned int old_error_count = error_count;

	struct variable_information seminib;
	Message object("Semaphore","ޥե");

	Load(num, "_kernel_tmax_semid");
	if(num < 1)
		return true;

	seminib = container->get_variable_information("_kernel_seminib_table");
	VerboseMessage("% object : % items\n","%֥ : % \n") << object << num;
	for(id = 1; id <= num; id++)
	{
		set_banner(parameter, object, SEMAPHORE, id);

			//attribute validation check
		LoadVal(work, seminib.address, "semaphore_initialization_block::sematr");
		if((work & ~0x1) != 0)
			notify(STANDARD,
				Message("Illegal attribute (It should be (TA_TFIFO||TA_TPRI)).",
						"(TA_TFIFO||TA_TPRI)ʳ°ꤵƤ"));

			//maxcnt < isemcnt
		LoadVal(work, seminib.address, "semaphore_initialization_block::maxsem");
		LoadVal(work2,seminib.address, "semaphore_initialization_block::isemcnt");
		if(work < work2)
			notify(STANDARD,
				Message("Initial count[%] is greater than the maximum count[%] of this semaphore",
						"[%][%]ĶƤ") << work2 << work);

		if(work == 0)
			notify(STANDARD,
				Message("Maximum count must be greater than zero.",
						"ޥեκ祫Ȥ1ʾǤʤФޤ"));

		seminib.address += seminib.size;
	}

	return old_error_count == error_count;
}


bool ConfigurationChecker::check_eventflagblock(Directory & parameter, filecontainer * container)
{
	unsigned int id;
	unsigned int num;
	unsigned int work;

	unsigned int old_error_count = error_count;
	struct variable_information inib;
	Message object("Event flag","٥ȥե饰");

	Load(num, "_kernel_tmax_flgid");
	if(num < 1)
		return true;

	inib = container->get_variable_information("_kernel_flginib_table");
	VerboseMessage("% object : % items\n","%֥ : % \n") << object << num;
	for(id = 1; id <= num; id++)
	{
		set_banner(parameter, object, EVENTFLAG, id);
			
			//attribute validation check
		LoadVal(work, inib.address, "eventflag_initialization_block::flgatr");
		if((work & ~0x7) != 0)
			notify(STANDARD,
				Message("Illegal attribute value [0x%]",
						"° [0x%]") << setbase(16) << (work & ~0x7));

		if((work & 0x2) != 0)
			notify(TOPPERS,		//¸ľʤΤRESTRICTEDˤʤ
				Message("Attribute TA_WMUL is not supported in current version.",
						"TA_WMULϥݡȳ"));
	
		inib.address += inib.size;
	}

	return old_error_count == error_count;
}


bool ConfigurationChecker::check_dataqueueblock(Directory & parameter, filecontainer * container)
{
	unsigned int id;
	unsigned int num;
	unsigned int work;
	unsigned int dtq;

	unsigned int old_error_count = error_count;
	struct variable_information inib;
	Message object("Data queue","ǡ塼");

	Load(num, "_kernel_tmax_dtqid");
	if(num < 1)
		return true;

	inib = container->get_variable_information("_kernel_dtqinib_table");
	VerboseMessage("% object : % items\n","%֥ : % \n") << object << num;
	for(id = 1; id <= num; id++)
	{
		set_banner(parameter, object, DATAQUEUE, id);

			//attribute validation check
		LoadVal(work, inib.address, "dataqueue_initialization_block::dtqatr");
		if((work & ~0x1) != 0)
			notify(STANDARD,
				Message("Illegal attribute value [0x%]",
						"° [0x%]") << setbase(16) << (work & ~0x1));

		LoadVal(work, inib.address, "dataqueue_initialization_block::dtqcnt");
		LoadVal(dtq, inib.address, "dataqueue_initialization_block::dtq");
		if(work != 0 && dtq == 0)
			notify(TOPPERS,
				Message("Dataqueue buffer should not be NULL", "ǡ塼ΥХåեNULL"));

		inib.address += inib.size;
	}

	return old_error_count == error_count;
}


bool ConfigurationChecker::check_mailboxblock(Directory & parameter, filecontainer * container)
{
	unsigned int id;
	unsigned int num;
	unsigned int work;
	int maxpri;
	int minpri;

	unsigned int old_error_count = error_count;
	struct variable_information inib;
	Message object("Mailbox","᡼ܥå");

	Load(num, "_kernel_tmax_mbxid");
	if(num < 1)
		return true;
	
	inib = container->get_variable_information("_kernel_mbxinib_table");
	maxpri = container->get_variable_information("TMAX_MPRI").value;
	minpri = container->get_variable_information("TMIN_MPRI").value;

	VerboseMessage("% object : % items\n","%֥ : % \n") << object << num;
	for(id = 1; id <= num; id++)
	{
		set_banner(parameter, object, MAILBOX, id);

			//attribute validation check
		LoadVal(work, inib.address, "mailbox_initialization_block::mbxatr");
		if((work & ~0x3) != 0)
			notify(STANDARD,
				Message("Illegal attribute value [0x%]",
						"° [0x%]") << setbase(16) << (work & ~0x3));
	
			//mailbox message priority check
		LoadVal(work, inib.address, "mailbox_initialization_block::maxmpri");
		if( container->get_variable_information("mailbox_initialization_block::maxmpri").size == 2)
			work = (int)((short)work);

		if((signed)work < 0)
			notify(STANDARD,
				Message("Priority must not be a negative number.","ͥ٤"));

		if((signed)work < minpri)
			notify(STANDARD,
				Message("Message priority should be greater than or equal to %.",
						"åͥ٤%ʾǤʤФޤ") << minpri);

		if((signed)work > maxpri)
			notify(STANDARD,
				Message("Message priority should be less than or equal to %.",
						"åͥ٤%ʲǤʤФޤ") << maxpri);

		inib.address += inib.size;
	}

	return old_error_count == error_count;
}

bool ConfigurationChecker::check_fixed_memorypoolblock(Directory & parameter, filecontainer * container)
{
	unsigned int id;
	unsigned int num;
	unsigned int work;
	unsigned int limit;

	unsigned int old_error_count = error_count;
	struct variable_information inib;
	Message object("Fixed size memory pool","Ĺס");

	Load(num, "_kernel_tmax_mpfid");
	if(num < 1)
		return true;

	inib = container->get_variable_information("_kernel_mpfinib_table");
	VerboseMessage("% object : % items\n","%֥ : % \n") << object << num;
	for(id = 1; id <= num; id++)
	{
		set_banner(parameter, object, FIXEDSIZEMEMORYPOOL, id);

			//attribute validation check
		LoadVal(work, inib.address, "fixed_memorypool_initialization_block::mpfatr");
		if((work & ~0x1) != 0)
			notify(STANDARD,
				Message("Illegal attribute value [0x%]","° [0x%]") << (work & ~0x1));

			//֥å0
		LoadVal(work, inib.address, "fixed_memorypool_initialization_block::mpf");
		LoadVal(limit, inib.address, "fixed_memorypool_initialization_block::limit");
		if(work == limit)
			notify(STANDARD,
				Message("blkcnt should be a non-zero value.","֥å0Ǥ"));

			//֥å0
		LoadVal(work, inib.address, "fixed_memorypool_initialization_block::blksz");
		if(work == 0)
			notify(STANDARD,
				Message("blksz should be a non-zero value.","֥å0Ǥ"));

			//Хåեɥ쥹0
		LoadVal(work, inib.address, "fixed_memorypool_initialization_block::mpf");
		if(work == 0)
			notify(TOPPERS,
				Message("buffer address is a NULL pointer.","Хåեɥ쥹NULLݥ󥿤ˤʤäƤޤ"));

		inib.address += inib.size;
	}

	return old_error_count == error_count;
}


bool ConfigurationChecker::check_cyclic_handlerblock(Directory & parameter, filecontainer * container)
{
	unsigned int id;
	unsigned int num;
	unsigned int work;
	unsigned int maxreltim;

	unsigned int old_error_count = error_count;
	struct variable_information inib;
	Message object("Cyclic handler","ϥɥ");

	Load(num, "_kernel_tmax_cycid");
	if(num < 1)
		return true;

	inib = container->get_variable_information("_kernel_cycinib_table");
	VerboseMessage("% object : % items\n","%֥ : % \n") << object << num;
	for(id = 1; id <= num; id++)
	{
		set_banner(parameter, object, CYCLICHANDLER, id);

			//attribute validation check
		LoadVal(work, inib.address, "cyclic_handler_initialization_block::cycatr");
		if((work & ~0x7) != 0)
			notify(STANDARD,
				Message("Illegal attribute value [0x%]","° [0x%]") << (work & ~0x1));

		if((work & 0x4) != 0)
			notify(TOPPERS,		//¸ʤΤRESTRICTEDˤʤ
				Message("TA_PHS is not supported in this kernel.","TA_PHSϥݡȳ"));

			// °ͤ TA_ASM ޤޤƤ
		if((work & 0x1) != 0)
			notify( RESTRICTED,
				Message("TOPPERS/JSP Kernel never minds the flag 'TA_ASM'.",
						"TOPPERS/JSPͥƤε¸TA_ASM򥵥ݡȤȤϸ¤ʤ"));

			//RELTIMǤɽϰˤ뤫ɤΥå
		maxreltim = container->get_variable_information("TMAX_RELTIM").value;
		LoadVal(work, inib.address, "cyclic_handler_initialization_block::cyctim");
		if(work > maxreltim)
			notify(STANDARD,
				Message("The cyclic object has a period (%) that exceeds the maximum period (%)",
						"ư(%)ɽǽл֤ϰ(%)ĶƤޤ") << work << maxreltim);

			//ư0ǤʤȤΥå
		if(work == 0)
			notify(STANDARD,
				Message("The cyclic object has a ZERO period.",
						"ư0ˤʤäƤޤ"));

		LoadVal(work, inib.address, "cyclic_handler_initialization_block::cycphs");
		if(work > maxreltim)
			notify(STANDARD,
				Message("The cyclic object has a initial delay (%) that exceeds the maximum period (%)",
						"ư(%)ɽǽл֤ϰ(%)ĶƤޤ") << work << maxreltim);

		inib.address += inib.size;
	}

	return old_error_count == error_count;
}

bool ConfigurationChecker::check_interrupt_handlerblock(Directory & parameter, filecontainer * container)
{
	unsigned int id;
	unsigned int num;
	unsigned int work;

	unsigned int old_error_count = error_count;
	struct variable_information inib;
	Message object("Interrupt handler","ߥϥɥ");

	Load(num, "_kernel_tnum_inhno");
	if(num == 0)
		return true;

	inib = container->get_variable_information("_kernel_inhinib_table");
	VerboseMessage("% object : % items\n","%֥ : % \n") << object << num;
	for(id = 0; id < num; id++)
	{
		set_banner(parameter, object, INTERRUPTHANDLER, id);

			//attribute validation check
		LoadVal(work, inib.address, "interrupt_handler_initialization_block::inhatr");
		if((work & 0x1) != 0)
			notify(STANDARD,
				Message("The attribute can take only TA_HLNG|TA_ASM",
						"TA_HNLG|TA_ASMʳ°Ǥޤ"));

			// °ͤ TA_ASM ޤޤƤ
		if((work & 0x1) != 0)
			notify(RESTRICTED,
				Message("TOPPERS/JSP Kernel never minds the flag 'TA_ASM'.",
						"TA_ASMѤƤ"));

			// ưϥå
		LoadVal(work, inib.address, "interrupt_handler_initialization_block::inthdr");
		if(work == 0)
			notify(RESTRICTED,
				Message("NULL pointer is specified as an inthdr address.",
						"ߥϥɥϤNULLǤ"));

		inib.address += inib.size;
	}

	return old_error_count == error_count;
}

bool ConfigurationChecker::check_exception_handlerblock(Directory & parameter, filecontainer * container)
{
	unsigned int id;
	unsigned int num;
	unsigned int work;

	unsigned int old_error_count = error_count;
	struct variable_information inib;
	Message object("Exception handler","㳰ϥɥ");

	Load(num, "_kernel_tnum_excno");
	if(num == 0)
		return true;

	inib = container->get_variable_information("_kernel_excinib_table");
	VerboseMessage("% object : % items\n","%֥ : % \n") << object << num;
	for(id = 0; id < num; id++)
	{
		set_banner(parameter, object, EXCEPTIONHANDLER, id);

			//attribute validation check
		LoadVal(work, inib.address, "cpu_exception_handler_initialization_block::excatr");
		if((work & 0x1) != 0)
			notify(STANDARD,
				Message("The attribute can take only TA_HLNG|TA_ASM",
						"TA_HNLG|TA_ASMʳ°Ǥޤ"));

			// °ͤ TA_ASM ޤޤƤ
		if((work & 0x1) != 0)
			notify(RESTRICTED,
				Message("TOPPERS/JSP Kernel never minds the flag 'TA_ASM'.",
						"TOPPERS/JSPͥƤε¸TA_ASM򥵥ݡȤȤϸ¤ʤ"));

			// ưϥå
		LoadVal(work, inib.address, "cpu_exception_handler_initialization_block::exchdr");
		if(work == 0)
			notify(RESTRICTED,
				Message("NULL pointer is specified as an exchdr address.",
						"㳰ϥɥϤNULLǤ"));

		inib.address += inib.size;
	}

	return old_error_count == error_count;
}


//------------------------------------------------------
#undef LoadVal
#undef Load

void ConfigurationChecker::parseOption(Directory & parameter)
{
	string loadmodule;
	string work;

	if(findOption(parameter,"h","help"))
	{
		cout << endl << Message(
			"Configuration checker - option\n"
			"  -m, --module=filename : Specify the load module (essential option)\n"
			"  -cs, --script=filename  : Specify the checker script file\n"
			"  -cl, --checklevel=level : Specify one of the check levels below \n"
			"    l(azy)       : Minimum check will be performed.\n"
			"    s(tandard)   : includes some ITRON Standard check items.\n"
			"    t(oppers)    : checks whether it meets TOPPERS/JSP restrictions\n"
			"    r(estricted) : All of check items will be performed.\n",
			"ե졼å - ץ\n"
			"  -m, --module=ե̾ : ɥ⥸塼̾ꤷޤ (ɬܹ)\n"
			"  -cs, --script=ե̾  : åץȤꤷޤ\n"
			"  -cl, --checklevel=level : Specify one of the check levels below \n"
			"    l(azy)       : Ǿ¤ΥåΤߤԤޤ\n"
			"    s(tandard)   : ITRONͤϰϤǥåԤޤ\n"
			"    t(oppers)    : TOPPERS/JSPͥ¤Ȥǧޤ\n"
			"    r(estricted) : ¸ޤƤΥåܤ»ܤޤ\n");
		return;
	}

	if(findOption(parameter,"m","module",&loadmodule) || findOption(parameter,DEFAULT_PARAMETER,NULL,&loadmodule))
	{
		if(findOption(parameter,"s","source"))
			ExceptionMessage("Configuration checker can not execute while Configurator executes","ե졼ȥåƱ˵ưǤޤ").throwException();

		parameter["/file/loadmodule"] = loadmodule;
		activateComponent(false);
	}

	if(!findOption(parameter,"cs","script",&work))
		work = loadmodule.substr(0,loadmodule.find_last_of('.')) + ".chk";
	parameter["/file/checkerscript"] = work;

	if(findOption(parameter,"obj","load-object",&work))
	{
		if(work.empty())
			work.assign("kernel_obj.dat");

		fstream f(work.c_str(), ios::in|ios::binary);
		if(f.is_open())
		{
			parameter["/object"].Load(&f);
			f.close();
		}else
			ExceptionMessage(" Failed to open the file '%' for storing object definitions"," ֥ȾǼ뤿Υե(%)ޤ") << work << throwException;
	}

	if(findOption(parameter,"cl","checklevel",&work))
	{
		current_level = UNKNOWN;

		if(work.compare("lazy") == 0 || work[0] == 'l')
			current_level = LAZY;
		if(work.compare("standard") == 0 || work[0] == 's')
			current_level = STANDARD;
		if(work.compare("toppers") == 0 || work[0] == 't')
			current_level = TOPPERS;
		if(work.compare("restricted") == 0 || work[0] == 'r')
			current_level = RESTRICTED;

		if(current_level == UNKNOWN)
			ExceptionMessage(" Unknown check level [%] specified"," ̵ʥå٥ [%]") << work << throwException;
	}else
		current_level = NORMAL;

	checkOption(parameter, "cpu", "cpu");
	checkOption(parameter, "system", "system");
}

void ConfigurationChecker::body(Directory & parameter)
{
	filecontainer * container;
	bool result = true;

		/* ͥ٤ι⤤顼оݤ */
	current_level = static_cast<enum tagCheckLevel>(static_cast<int>(current_level) * 2 - 1);

	container = filecontainer::get_instance();
	container->attach_infofile(parameter["/file/checkerscript"].toString().c_str());
	container->attach_module(parameter["/file/loadmodule"].toString().c_str());

	if(VerboseMessage::getVerbose())
	{
		cout << Message("  Target architecture : ","  åȥƥ : ")
			 << container->get_architecture() << endl;
	}

	error_count = 0;
	result &= check_taskblock(parameter,container);
	result &= check_semaphoreblock(parameter,container);
	result &= check_eventflagblock(parameter,container);
	result &= check_dataqueueblock(parameter,container);
	result &= check_mailboxblock(parameter,container);
	result &= check_fixed_memorypoolblock(parameter,container);
	result &= check_cyclic_handlerblock(parameter,container);
	result &= check_interrupt_handlerblock(parameter,container);
	result &= check_exception_handlerblock(parameter,container);

	if(!result)
		ExceptionMessage("Total % errors found in current configuration.\n","%ĤΥ顼Фޤ\n") << error_count << throwException;

	VerboseMessage("No error found in current configuration\n","˰۾ϤޤǤ\n");
}
