/*
 *  TOPPERS/JSP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Just Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 * 
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation 
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 * 
 *  @(#) $Id: jsp_parser.cpp,v 1.3 2003/11/10 07:51:15 monaka Exp $
 */

// $Header: /cvsroot/toppers-utils/if40/cfg/jsp/jsp_parser.cpp,v 1.3 2003/11/10 07:51:15 monaka Exp $

#include <stdarg.h>

#include "jsp/jsp_defs.h"
#include "base/parser.h"
#include "base/mpstrstream.h"
#include <set>
#include <map>

using namespace std;

class CoreParser : public ParserComponent
{
protected:
    virtual void parseOption(Directory &);  
public:
	CoreParser(void);
	virtual bool parse(Parser & p, Directory & container);
} instance_of_Parser;

CoreParser::CoreParser(void)
{
	bootBanner().assign("=== TOPPERS/JSP Kernel Configurator ver.9.3 (for JSP rel 1.4) ===");
}

void CoreParser::parseOption(Directory & container)
{
	ParserComponent::parseOption(container);

	if(findOption(container,"h","help"))
	{
        cerr << Message(
			"  -obj, --dump-object=filename : Dump the object tree into the file specified\n"
            "  -ao, --assign-order=order : Specify the order of automatic ID assignment\n"
			"    You can use three terms below as ordering rule.\n"
            "     alphabetic (in alphabetic order)\n"
			"     fcfs       (in arrival order [as default])\n"
			"     reverse    (reverse the order)\n",
			"  -obj, --dump-object=ե̾ : ꤷե˥֥ȾϤޤ\n"
            "  -ao, --assign-order= : ưIDγꤷޤ\n"
			"    ϼ3Ĥȹ礻ǻꤷޤ.\n"
            "     alphabetic (ABC), fcfs ( [ǥե]), reverse (ս)\n");
        return;
	}

	checkOption(container,"cpu","cpu");
	checkOption(container,"system","system");
}

static int displayHandler(Directory & container, const char * category, const char * format)
{
	Directory * node;
	Directory * scope;

	node = container.findChild(OBJECTTREE,category,NULL);
	if(node == 0 || node->size() == 0)
		return 0;

	VerboseMessage("Handler assignment list [%]\n","ϥɥɽ [%]\n") << category;

	scope = node->getFirstChild();
	while(scope != 0)
	{
		VerboseMessage::getStream() << scope->format(format);
		scope = scope->getNext();
	}

	return node->size();
}

bool CoreParser::parse(Parser & p, Directory & container)
{
	Token token;
	string work;
    bool isParseErrorOccured;
	multimap<int, const char *> sorter;
	multimap<int, const char *>::iterator scope;
    enum Common::tagAssignmentOrder order;

    if(findOption(container, "cpu","cpu", &work))
        container["/cpu"] = work;
    
	try{
		isParseErrorOccured = false;
		do {
			p.getToken(token);
			if(token == Token::IDENTIFIER)
			{
					/* 顼Ƥ³Τǡ
						ȤФƤƤɤɤ³ */
				try 
				{	this->parseStaticAPI(p, container, token);	}
				catch(...)
				{	isParseErrorOccured = true;					}

			}
		}while(token != Token::EOS);

		p.setLogBuffer(NULL);

		if(isParseErrorOccured)
			ExceptionMessage(ExceptionMessage::FATAL,"The configuration process was aborted due to occurence of parse error","ʸϤ˼ԤǤޤ").throwException();

		if(container[OBJECTTREE "/" TASK].size() == 0)
			ExceptionMessage("Kernel requires one or more task objects.","֥Ȥޤ").throwException();

		sorter.insert(pair<int, const char *>(container[OBJECTTREE "/" TASK].size(), TASK));
		sorter.insert(pair<int, const char *>(container[OBJECTTREE "/" SEMAPHORE].size(), SEMAPHORE));
		sorter.insert(pair<int, const char *>(container[OBJECTTREE "/" EVENTFLAG].size(), EVENTFLAG));
		sorter.insert(pair<int, const char *>(container[OBJECTTREE "/" DATAQUEUE].size(), DATAQUEUE));
		sorter.insert(pair<int, const char *>(container[OBJECTTREE "/" MAILBOX].size(), MAILBOX));
		sorter.insert(pair<int, const char *>(container[OBJECTTREE "/" FIXEDSIZEMEMORYPOOL].size(), FIXEDSIZEMEMORYPOOL));
		sorter.insert(pair<int, const char *>(container[OBJECTTREE "/" CYCLICHANDLER].size(), CYCLICHANDLER));

        order = Common::parseOrder(getOption(container, "ao", "assign-order"));
		scope = sorter.begin();
		while(scope != sorter.end())
		{
			if( (*scope).first != 0 )
                Common::assignID(container, (*scope).second, OBJECTTREE, order);
			++ scope;
		}

		displayHandler(container, INTERRUPTHANDLER, "  $@ : $(inthdr)\n");
		displayHandler(container, EXCEPTIONHANDLER, "  $@ : $(exchdr)\n");
		displayHandler(container, INITIALIZER, "  $@ : $(inirtn)($(exinf))\n");
		displayHandler(container, TERMINATOR, "  $@ : $(terrtn)($(exinf))\n");

		return true;
	}
	catch(Exception &)
	{		throw;		}
}

/* ======================================================================== */

DECLARE_DIRECTIVE(assigner,"assign_id")
{
	Token name;
	Token value;

	p.getToken(name, Token::IDENTIFIER);
	p.getToken(value, Token::INTEGER);

	if(value.value <= 0)
		ExceptionMessage("Cannot assign an ID number less or equal to 0.","0ʲIDֹꤹ뤳ȤϤǤޤ").throwException();

	*container.openChild("/","identifier",name.c_str(),NULL) = value.value;
}

DECLARE_DIRECTIVE(parameter,"parameter")
{
	Token token;
	string key;
	string::size_type i,j;

	do {
		p.getToken(token);
		if(!(token == Token::STRINGLITERAL))
			ExceptionMessage("parameter pragma requires additional parameter put in '\"...\"'","parameterץ饰ޤˤ\"...\"ǳ줿ɲåѥ᡼ɬפǤ").throwException();

		token.chopLiteral();

		if(token[0] == '-')
		{
			i = token.find_first_of('=');
			j = token.find_first_not_of('-');
			if(i == j || j == string::npos)
                ExceptionMessage(ExceptionMessage::FATAL,"Wrong option [%]","ʥץ [%]") << token << throwException;

			if(i != string::npos)
			{
				key = DEFAULT_PARAMETER;
				*container.openChild(string("/Parameter/") + token.substr(1, i-1)) = token.substr(i+1);
			}
			else
			{
				key = token.substr(1);
				*container.openChild(string("/Parameter/") + key) = string("");
			}
		}else
			*container.openChild(string("/Parameter/") + key) = token;

		p.getToken(token);
	} while(token.compare(",") == 0);

	p.putBack(token);
}


/* ======================================================================== */
class ConfigurationFileGenerator : public Component
{
protected:
	bool backward_compatible;
	bool libstyle_include;
	bool restrict_zero;
	bool def_prototype;

	enum tagDefinition { HEADER=1, TNUM=2, BUFFER=4, DEFINITION=8, CONTROLBLOCK=16, SHORT_ID=32, TNUMNO=64, INIT=128, PROTOTYPE=256, INITLIST=512 };
	void createObjectDefinition(mpstrstream *, Directory &, int, ...);

	virtual void parseOption(Directory &);
	virtual void body(Directory &);
	virtual void onFatalExit(Directory &);

public:
	ConfigurationFileGenerator(void) : Component(CODEGENERATOR) {};
} instance_of_Configurator;

void ConfigurationFileGenerator::parseOption(Directory & parameter)
{
	string work;
	mpstrstream * stream;

	if(findOption(parameter,"h","help"))
	{
        cerr << endl << Message(
			"Kernel configuration file generator\n"
            "  -id, --kernel_id=filename  : Specify the name of ID assignment file\n"
            "  -cfg,--kernel_cfg=filename : Specify the name of kernel configuration file\n"
			"  -il, --include-libstyle    : Use #include <...> style for kernel headers\n"
			"  -oproto, --output-prototype: Output task prototype definitions in kernel_cfg\n"
			"  -z, --nonzero              : Do not output ZERO macro and related definitions\n"
            "  -1.3  : Generates kernel_cfg.c for TOPPERS/JSP rel 1.3\n",
			"ͥ빽ե\n"
            "  -id, --kernel_id=ե̾  : IDե̾ꤷޤ\n"
            "  -cfg,--kernel_cfg=ե̾ : ͥ빽ե̾ꤷޤ\n"
			"  -il, --include-libstyle      : ͥΥإåե<...>ηǽϤޤ\n"
			"  -oproto, --output-prototype  : Τʤɤkernel_cfg.cޤ\n"
			"  -z, --nonzero                : ZEROޥλѤ¤ޤ\n"
            "  -1.3  : TOPPERS/JSP rel 1.3 ѤΥե졼եޤ\n");
        return;
	}

		/*
		 *  ͥ륳ե졼̤Υե (kernel_cfg.c) ν
		 */
	if(!findOption(parameter,"cfg","kernel_cfg",&work))
		work.assign("kernel_cfg.c");

	stream = new mpstrstream(work);
	stream->createpart("header","include",TASK,SEMAPHORE,EVENTFLAG,DATAQUEUE,MAILBOX,FIXEDSIZEMEMORYPOOL,CYCLICHANDLER,INTERRUPTHANDLER,EXCEPTIONHANDLER,INITIALIZER,OBJECT_INITIALIZER,"others",NULL);
	parameter[KERNEL_CFG] = stream;


		/*
		 *  IDշ̤Υե (kernel_id.h) ν
		 */
	if(!findOption(parameter,"id","kernel_id",&work))
		work.assign("kernel_id.h");

	stream = new mpstrstream(work);
	stream->createpart("header","body","footer",NULL);
	(*stream)["header"] << "#ifndef KERNEL_ID_H\n#define KERNEL_ID_H\n\n";
	(*stream)["footer"] << "\n#endif /* KERNEL_ID_H */\n";
	parameter[KERNEL_ID] = stream;

		/* ¾ץ */
	restrict_zero = findOption(parameter,"z","zero");
	def_prototype = findOption(parameter,"oproto","output-prototype");
	libstyle_include = findOption(parameter,"il","include-libstyle");
	backward_compatible = findOption(parameter,"1.3","1.3");
	
	checkOption(parameter,"mcfg","minimize-cfg");
	checkOption(parameter,"ZERO","ZERO");
	checkOption(parameter,"obj", "dump-object");

	activateComponent();
}

void ConfigurationFileGenerator::onFatalExit(Directory & container)
{
	mpstrstream * stream;

		//㳰ǰ۾ｪλ顢եʤ
	stream = reinterpret_cast<mpstrstream *>(container[KERNEL_CFG].toPointer());
	if(stream != NULL)
		stream->disableFileoutput();

	stream = reinterpret_cast<mpstrstream *>(container[KERNEL_ID].toPointer());
	if(stream != NULL)
		stream->disableFileoutput();
}

void ConfigurationFileGenerator::createObjectDefinition(mpstrstream * out, Directory & container, int flag, ...)
{
	Directory * scope;
	const char * buffer;
	string work;

	map<int, Directory *> sorter;
	map<int, Directory *>::iterator psorter;

	va_list vl;
	int i;

	Directory & id = container[string(PREFIX "/") + container.getKey().c_str()];

	if(container.size() == 0 && findOption(container,"mcfg","minimize-cfg"))
	{
		Component::barrier(Component::EXTENSION,Message("Use of the extended option --minimize-cfg. Please use -ext option together.","--minimizeץϳĥǽǤѤˤ--extꤷƤ"));
		return;
	}
	va_start(vl, flag);


		//ɬפʾκ
	work = string(va_arg(vl, const char *));
	id["id"]  = work;
	for(i=0; i<(signed)work.size(); i++)
		work[i] = work[i] - 'a' + 'A';
	id["ID"]  = work;

	if((flag & SHORT_ID) != 0)
	{
		work = string(va_arg(vl, const char *));
		id["sid"]  = work;
		for(i=0; i<(signed)work.size(); i++)
			work[i] = work[i] - 'a' + 'A';
		id["SID"]  = work;
	}else
	{
		id["sid"] = id["id"].toString();
		id["SID"] = id["ID"].toString();
	}

	if(!backward_compatible)
		id["symbolprefix"] = "_kernel_";

	work = container.getKey();
	id["name"] = work;
	out->moveto(work);

		//֥Ƚؿɲ
	if((flag & INIT) != 0 && container.size() != 0)
		container["/" OBJECT_INITIALIZER].addChild(new Directory(work));

	(*out) << "\n\t/* " << (Message("Object initializer [%]","֥Ƚ֥å [%]") << work) << " */\n\n";

	if(((flag & HEADER) != 0) && backward_compatible)
	{
		(*out) << "#include ";
		if(libstyle_include)
			(*out) << '<' << work << ".h>\n\n";
		else
			(*out) << '\"' << work << ".h\"\n\n";
	}

	if((flag & TNUM) != 0)
	{
		(*out) << id.format("#define TNUM_$(ID)ID ") << container.size() << "\n\n";
		(*out) << id.format("const ID $(symbolprefix,)tmax_$(id)id = (TMIN_$(ID)ID + TNUM_$(ID)ID - 1);\n\n");
	}

	if((flag & TNUMNO) != 0)
	{
		(*out) << id.format("#define TNUM_$(ID)NO ") << container.size() << "\n\n";
		(*out) << id.format("const UINT $(symbolprefix,)tnum_$(id)no = TNUM_$(ID)NO;\n\n");
	}

		/* ID˥ (λϢ֤ꤷƤϤʤΤǡåϤʤ) */
	if((flag & (BUFFER|DEFINITION)) != 0)
	{
		const bool is_identifier = (flag & TNUMNO) == 0;

		i = 0;
		scope = container.getFirstChild();
		while(scope != 0)
		{
			if(is_identifier)
				sorter[scope->toInteger(i++)] = scope;
			else
				sorter[i++] = scope;

			scope = scope->getNext();
		}
	}

		//ץȥν
	if((flag & PROTOTYPE) != 0)
	{
		buffer = va_arg(vl, const char *);

		if(def_prototype)
		{
			set<string> func;
			set<string>::iterator pfunc;

			scope = container.getFirstChild();
			while(scope != NULL)
			{
				func.insert(scope->format(buffer));
				scope = scope->getNext();
			}

			pfunc = func.begin();
			while(pfunc != func.end())
			{
				(*out) << (*pfunc) << endl;
				++ pfunc;
			}
		}
	}

		//Хåեκ
	if((flag & BUFFER) != 0 && container.size() != 0)
	{
		buffer = va_arg(vl, const char *);
		psorter = sorter.begin();
		while(psorter != sorter.end())
		{
			(*out) << (*psorter).second->format(buffer) << '\n';
			++ psorter;
		}
		(*out) << '\n';
	}

		//֥å(xINIB)ν
	if((flag & DEFINITION) != 0)
	{
		buffer = va_arg(vl, const char *);
		if(container.size() != 0)
		{
			(*out) << id.format("const $(SID)INIB $(symbolprefix,)$(sid)inib_table[TNUM_$(ID)") << ( (flag & TNUMNO) != 0  ? "NO" : "ID") << "] = {\n";
			psorter = sorter.begin();
			while(psorter != sorter.end())
			{
				(*out) << "\t{" << (*psorter).second->format(buffer) << '}';
				++ psorter;
				if(psorter != sorter.end())
					(*out) << ',';
				(*out) << '\n';
			}
			(*out) << "};\n\n";
		}else
			if( !restrict_zero )
				(*out) << id.format("__ZERO(const $(SID)INIB,$(symbolprefix,)$(sid)inib_table);\n");
	}

	if((flag & INITLIST) != 0)
	{
		Directory * node;

			/* IDֹ򥽡 */
		sorter.clear();
		node = container.getFirstChild();
		while(node != NULL)
		{
			sorter.insert(pair<int,Directory*>(node->findChild("#order")->toInteger(), node));
			node = node->getNext();
		}

		*out << id.format("const ID $(symbolprefix,)$(sid)order_table[TNUM_$(ID)ID] = {");
		psorter = sorter.begin();
		while(psorter != sorter.end())
		{
			*out << psorter->second->toInteger();
			++ psorter;
			if(psorter != sorter.end())
				*out << ',';
		}
		*out << "};\n\n";
	}

		//֥å(xCB)ν
	if((flag & CONTROLBLOCK) != 0)
	{
		if(container.size() != 0)
			(*out) << id.format("$(SID)CB $(symbolprefix,)$(sid)cb_table[TNUM_$(ID)ID];\n\n");
		else
			if( !restrict_zero )
				(*out) << id.format("__ZERO($(SID)CB, $(symbolprefix,)$(sid)cb_table);\n\n");
	}
}

void ConfigurationFileGenerator::body(Directory & container)
{
	Directory * scope;
	mpstrstream * out;
	string work;

		/* kernel_id.hν */
	out = reinterpret_cast<mpstrstream *>(container(KERNEL_ID)->toPointer());
	out->moveto("body");
	scope = container("/identifier")->getFirstChild();
	while(scope != 0)
	{
		(*out) << "#define " << scope->getKey() << " " << scope->toInteger() << endl;
		scope = scope->getNext();
	}

		/* kernel_cfg.cν */
	out = reinterpret_cast<mpstrstream *>(container(KERNEL_CFG)->toPointer());
	out->moveto("header");
	(*out) << "/* Configured by [" << container["/Parameter/#cmdline"].toString() << "] */\n\n";

	if(!backward_compatible)
	{
		if(libstyle_include)
			(*out) <<	"#include <kernel_cfg.h>\n";
		else
			(*out) <<	"#include \"kernel_cfg.h\"\n";
	}

	if(!findOption(container,"id","kernel_id",&work))
		work.assign("kernel_id.h");
	(*out) << "#include \"" << work << "\"\n\n";

	if(backward_compatible)
		(*out) <<	"#if TKERNEL_PRVER >= 0x1011\n\n";


	(*out) <<	"#define CFG_INTHDR_ENTRY(inthdr) INTHDR_ENTRY(inthdr)\n"
				"#define CFG_EXCHDR_ENTRY(exchdr) EXCHDR_ENTRY(exchdr)\n"
				"#define CFG_INT_ENTRY(inthdr) INT_ENTRY(inthdr)\n"
				"#define CFG_EXC_ENTRY(exchdr) EXC_ENTRY(exchdr)\n";

	if(backward_compatible)
		(*out) <<	"\n#else\n\n"
					"#define CFG_INTHDR_ENTRY(inthdr) INTHDR_ENTRY(ENTRY(inthdr), inthdr)\n"
					"#define CFG_EXCHDR_ENTRY(exchdr) EXCHDR_ENTRY(ENTRY(exchdr), exchdr)\n"
					"#define CFG_INT_ENTRY(inthdr) ENTRY(inthdr)\n"
					"#define CFG_EXC_ENTRY(exchdr) ENTRY(exchdr)\n"
					"\n#endif\n";

	(*out) << "\n";

	if( !restrict_zero )
	{
		if(!findOption(container, "ZERO","ZERO",&work))
			work.assign("x y[0]");
		(*out) << "#if defined(_MSC_VER) || defined(__BORLANDC__)\n  #define __ZERO(x,y) _declspec(naked) void y(void){}\n#else\n  #define __ZERO(x,y) " << work << "\n#endif\n\n";
	}

		/* INCLUDE */
	out->moveto(INCLUDEFILE);
	scope = container(OBJECTTREE "/" INCLUDEFILE)->getFirstChild();
	(*out) << "\t/* " << Message("User specified include files","桼Υ󥯥롼ɥե") << "*/\n";
	while(scope != 0)
	{
		(*out) << "#include " << scope->toString() << endl;
		scope = scope->getNext();
	}
	(*out) << '\n';

		/* TASK */
    work.assign("$(tskatr), (VP_INT)($(exinf)), (FP)($(task)), INT_PRIORITY($(itskpri)), TROUND_VP($(stksz)), __stack_$@, $(texatr), (FP)($(texrtn))");
    if(container["/cpu"].toString().compare("nios32") == 0)
        work += ", $(hi_limit), $(lo_limit)";
	createObjectDefinition(out, container[OBJECTTREE "/" TASK], HEADER|TNUM|SHORT_ID|BUFFER|DEFINITION|CONTROLBLOCK|INIT|PROTOTYPE|INITLIST,"tsk", "t", "void $(task)(VP_INT exinf);", "static VP __stack_$@[TCOUNT_VP($(stksz))];",work.c_str());

		/* SEMAPHORE */
	createObjectDefinition(out, container[OBJECTTREE "/" SEMAPHORE], HEADER|TNUM|DEFINITION|CONTROLBLOCK|INIT,"sem", "$(sematr), $(isemcnt), $(maxsem)");

		/* EVENTFLAG */
	createObjectDefinition(out, container[OBJECTTREE "/" EVENTFLAG], HEADER|TNUM|DEFINITION|CONTROLBLOCK|INIT,"flg", "$(flgatr), $(iflgptn)");

		/* DATAQUEUE */
	createObjectDefinition(out, container[OBJECTTREE "/" DATAQUEUE], HEADER|TNUM|BUFFER|DEFINITION|CONTROLBLOCK|INIT,"dtq", "#if ($(dtqcnt)) > 0\n  static VP_INT __dataqueue_$@[$(dtqcnt)];\n#else\n  #define __dataqueue_$@ NULL\n#endif","$(dtqatr), $(dtqcnt), __dataqueue_$@");

		/* MAILBOX */
	createObjectDefinition(out, container[OBJECTTREE "/" MAILBOX], HEADER|TNUM|DEFINITION|CONTROLBLOCK|INIT,"mbx","$(mbxatr), $(maxmpri)");

		/* FIXEDSIZEMEMORYPOOL */
	createObjectDefinition(out, container[OBJECTTREE "/" FIXEDSIZEMEMORYPOOL], HEADER|TNUM|BUFFER|DEFINITION|CONTROLBLOCK|INIT,"mpf","static VP __fixedsize_memorypool_$@[TCOUNT_VP($(blksz)) * (($(blkcnt)))];","$(mpfatr), TROUND_VP($(blksz)), __fixedsize_memorypool_$@, (VP)((VB *)__fixedsize_memorypool_$@ + sizeof(__fixedsize_memorypool_$@))");

		/* CYCLICHANDLER */
	createObjectDefinition(out, container[OBJECTTREE "/" CYCLICHANDLER], HEADER|TNUM|DEFINITION|CONTROLBLOCK|INIT|PROTOTYPE,"cyc","void $(cychdr)(VP_INT exinf);","$(cycatr),$(exinf),(FP)($(cychdr)),$(cyctim),$(cycphs)");

		/* INTERRUPTHANDLER */
	createObjectDefinition(out, container[OBJECTTREE "/" INTERRUPTHANDLER], HEADER|BUFFER|TNUMNO|DEFINITION|INIT|PROTOTYPE,"inh","void $(inthdr)(void);","CFG_INTHDR_ENTRY($(inthdr));","$@,$(inhatr),(FP)CFG_INT_ENTRY($(inthdr))");

		/* EXCEPTIONHANDLER */
	createObjectDefinition(out, container[OBJECTTREE "/" EXCEPTIONHANDLER], HEADER|BUFFER|TNUMNO|DEFINITION|INIT|PROTOTYPE,"exc","void $(exchdr)(VP p_excinf);","CFG_EXCHDR_ENTRY($(exchdr));","$@,$(excatr),(FP)CFG_EXC_ENTRY($(exchdr))");


	if(!backward_compatible)
		work.assign("_kernel_");
	else
		work.assign("");


		/* ֥Ƚ롼 */
	out->moveto(OBJECT_INITIALIZER);
	(*out) << "\t/* " << Message("Object initialization routine","֥Ƚ롼") << " */\n\n";
	(*out) << "void\n" << work << "object_initialize(void)\n{\n";
	scope = container["/" OBJECT_INITIALIZER].getFirstChild();
	while(scope != 0)
	{
		(*out) << '\t' << work << scope->toString() << "_initialize();\n";
		scope = scope->getNext();
	}
	(*out) << "}\n\n";


		/* ϥɥ */
	out->moveto(INITIALIZER);
	(*out) << "\t/* " << Message("Initialization handler","ϥɥ鵯ư롼") << " */\n\n";
	(*out) << "void\n" << work << "call_inirtn(void)\n{\n";
	scope = container[OBJECTTREE "/" INITIALIZER].getFirstChild();
	while(scope != 0)
	{
		(*out) << scope->format("\t$(inirtn)( (VP_INT)($(exinf)) );\n");
		scope = scope->getNext();
	}
	(*out) << "}\n\n";

		/* λϥɥ */
	scope = container[OBJECTTREE "/"  TERMINATOR].getLastChild();
	if(!backward_compatible) {
		(*out) << "void\n" << work << "call_terrtn(void)\n{\n";
		while(scope != 0)
		{
			(*out) << scope->format("\t$(terrtn)( (VP_INT)($(exinf)) );\n");
			scope = scope->getPrev();
		}
		(*out) << "}\n\n";
	}
	else {
		if(scope->size() != 0)
			ExceptionMessage("VATT_TER is not supported for TOPPERS/JSP rel 1.3 or earlier.",
							 "VATT_TERTOPPERS/JSP rel 1.3ǤϥݡȤޤ").throwException();
	}

	out->moveto("others");
	(*out) << "TMEVTN	" << work << "tmevt_heap[TNUM_TSKID + TNUM_CYCID];\n\n";

	container["/object_initializer"].erase();

	string filename;
	if(findOption(container,"obj","dump-object",&filename))
	{
		if(filename.empty())
			filename.assign("kernel_obj.dat");

		fstream f(filename.c_str(),ios::out|ios::binary);
		if(f.is_open())
		{
			container[OBJECTTREE].Store(&f);
			f.close();
		}else
			ExceptionMessage(" Failed to open the file % for storing object definitions"," ֥ȾǼ뤿Υե(%)ޤ") << filename << throwException;
	}
}


/* ======================================================================== */
